import pytest
from fastai import *
from fastai.vision import *

def bbox2pic(corners, size):
    pic = torch.zeros(1, size,size)
    pic[0, corners[0]:corners[2], corners[1]:corners[3]] = 1.
    return Image(pic)

def points2pic(points, size):
    points = points.clamp(min=0,max=size-1)
    pic = torch.zeros(1, size,size)
    for p in points: pic[0, max(0,int(p[0])-1):min(size, int(p[0])+1),max(0,int(p[1])-1):min(size, int(p[1])+1)] = 1.
    return Image(pic)

def test_points_data_aug():
    "Check that ImagePoints get changed with their input Image."
    points = torch.randint(0,64, ((5,2)))
    img = points2pic(points, 64)
    pnts = ImagePoints(FlowField((64,64), points.float()))
    tfms = get_transforms()
    tfm_x = apply_tfms(tfms[0], img, size=64, mode='nearest')
    tfm_y = apply_tfms(tfms[0], pnts, do_resolve=False, size=64)
    new_pnts = scale_flow(FlowField(tfm_y.size, tfm_y.data), to_unit=False).flow.round()
    fail = False
    for p in new_pnts.round():
        if tfm_x.data[0, max(0,int(p[0])-1):min(int(p[0])+2,64), max(0,int(p[1])-1):min(int(p[1])+2,64)].sum() < 0.8: 
            fail = True
    assert not fail

def test_bbox_data_aug():
    "Check that ImagePoints get changed with their input Image."
    pick_box = True
    while pick_box:
        points = torch.randint(5,59, ((2,2)))
        #Will fail if box to close to the border
        corners = torch.cat([points.min(0)[0], points.max(0)[0]])
        pick_box = (corners[2:] - corners[:2]).min() < 2
    img = bbox2pic(corners, 64)
    bbox = ImageBBox.create([list(corners)], 64, 64)
    tfms = get_transforms()
    tfm_x = apply_tfms(tfms[0], img, size=64, mode='nearest', padding_mode='zeros')
    tfm_y = apply_tfms(tfms[0], bbox, do_resolve=False, size=64, padding_mode='zeros')
    new_bb = ((tfm_y.data + 1) * 32)
    mask = (tfm_x.data[0] > 0.5).nonzero()
    if len(mask) == 0:
        assert (new_bb[0][2:] - new_bb[0][:2]).min() < 1
    else:
        img_bb = torch.cat([mask.min(0)[0], mask.max(0)[0]])
        assert (new_bb - img_bb.float()).abs().max() < 2


import pytest
from fastai import *
from fastai.text import *

pytestmark = pytest.mark.integration

def read_file(fname):
    tokens = []
    with open(fname, 'r') as f:
        for line in f.readlines():
            tokens.append(np.array(line[:-1].split(' ')[:-1]))
    return tokens

def prep_human_numbers():
    path = untar_data(URLs.HUMAN_NUMBERS)
    trn_tokens, val_tokens = read_file(path/'train.txt'), read_file(path/'valid.txt')
    np.save(path/'train_tok.npy', trn_tokens)
    np.save(path/'valid_tok.npy', val_tokens)
    return path

@pytest.fixture(scope="module")
def learn():
    path = prep_human_numbers()
    data = TextLMDataBunch.from_tokens(path)
    learn = RNNLearner.language_model(data, emb_sz=100, nl=1, drop_mult=0.)
    learn.fit_one_cycle(4, 1e-2)
    return learn

def test_val_loss(learn):
    assert learn.validate()[1] > 0.3



import pytest
from fastai import *
from fastai.collab import *

pytestmark = pytest.mark.integration

@pytest.fixture(scope="module")
def learn():
    ratings = URLs.get_movie_lens()
    ratings.head()
    series2cat(ratings, 'userId','movieId')
    learn = get_collab_learner(ratings, n_factors=50, pct_val=0.2, min_score=0., max_score=5.)
    learn.fit_one_cycle(3, 5e-3, wd=0.1)
    return learn

def test_val_loss(learn):
    assert learn.validate()[0] < 0.75



from fastai.text.data import LanguageModelLoader, TextDataset, Tokenizer
import pandas as pd
import numpy as np


class DummyTokenizer(Tokenizer):

    def process_all(self, texts):
        return [t.split() for t in texts]

def test_should_load_backwards_lm():
    # GIVEN
    df = pd.DataFrame([{0: 0, "text": "fast ai is a cool project"}, {0: 0, 'text': "hello world"}])
    text_ds = TextDataset.from_df('/tmp/', df, label_cols=[0], txt_cols=["text"], min_freq=0, tokenizer=DummyTokenizer())
    # WHEN
    lml = LanguageModelLoader(text_ds, bs=1, backwards=True)
    # THEN
    batch = lml.get_batch(0, 70)

    as_text = [text_ds.vocab.itos[x] for x in batch[0]]
    np.testing.assert_array_equal(as_text[:2], ["world", "hello"])



import pytest
from fastai import *
from fastai.text import *

def test_rules():
    assert deal_caps("I'm suddenly SHOUTING FOR NO REASON!") == "i'm suddenly  xxup shouting  xxup for no  xxup reason!"
    assert fix_html("Some HTML&nbsp;text<br />") == "Some HTML& text\n"
    assert replace_rep("I'm so excited!!!!!!!!") == "I'm so excited xxrep 8 ! "
    assert replace_wrep("I've never ever ever ever ever ever ever ever done this.") == "I've never  xxwrep 7 ever  done this."
    assert rm_useless_spaces("Inconsistent   use  of     spaces.") == "Inconsistent use of spaces."
    assert spec_add_spaces('I #like to #put #hashtags #everywhere!') == "I  # like to  # put  # hashtags  # everywhere!"

def test_tokenize():
    texts = ['one two three four', 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua.']
    tokenizer = Tokenizer(BaseTokenizer)
    toks = tokenizer.process_all(texts)
    assert toks[0] == ['one', 'two', 'three', 'four']
    assert toks[1][:5] == ['lorem', 'ipsum', 'dolor', 'sit', 'amet,']

    


import pytest
from fastai import *
from fastai.tabular import *

pytestmark = pytest.mark.integration

@pytest.fixture(scope="module")
def learn():
    path = untar_data(URLs.ADULT_SAMPLE)
    df = pd.read_csv(path/'adult.csv')
    tfms = [FillMissing, Categorify]
    train_df, valid_df = df[:1024].copy(),df[1024:1260].copy()
    dep_var = '>=50k'
    cat_names = ['workclass', 'education', 'marital-status', 'occupation', 'relationship', 'race', 'sex', 'native-country']
    data = TabularDataBunch.from_df(path, train_df, valid_df, dep_var, tfms=tfms, cat_names=cat_names)
    learn = get_tabular_learner(data, layers=[200,100], emb_szs={'native-country': 10}, metrics=accuracy)
    learn.fit_one_cycle(3, 1e-2)
    return learn

def test_accuracy(learn):
    assert learn.validate()[1] > 0.75
